float4x4 gWorld;
float4x4 gView;
float4x4 gViewInverse;
float4x4 gProj;
float4x4 gProjInverse;

texture2D gNormalMap;
texture2D gDepthMap;
texture2D gPositionXYMap;
texture2D gMaterialMap;

texture2D gDiffuseMap;
texture   gNormalTexture;

sampler DiffuseSampler = sampler_state
{
	Texture = <gDiffuseMap>;
	
	AddressU = Wrap;
	AddressV = Wrap;
	
	MagFilter = Linear;
	MinFilter = Linear;
	MipFilter = Linear;
};

sampler NormalSampler = sampler_state
{
	Texture = <gNormalTexture>;
	
	AddressU = Wrap;
	AddressV = Wrap;
	
	MagFilter = Linear;
	MinFilter = Linear;
	MipFilter = Linear;
};

// rendertargets
sampler SceneMaterialSampler = sampler_state
{
	Texture = <gMaterialMap>;
	MagFilter = Point;
	MinFilter = Point;
};

sampler SceneNormalSampler = sampler_state
{
	Texture = <gNormalMap>;
	MagFilter = Point;
	MinFilter = Point;
};

sampler ScenePositionXYSampler = sampler_state
{
	Texture = <gPositionXYMap>;
	MagFilter = Point;
	MinFilter = Point;
};

sampler SceneDepthSampler = sampler_state
{
	Texture = <gDepthMap>;
	MagFilter = Point;
	MinFilter = Point;
};

struct InputBuildVS
{
	float3 posIn  : POSITION0;
	float2 texIn  : TEXCOORD0;
	float3 normIn : NORMAL0;
	float3 tanIn  : TANGENT0;
	float3 binIn  : BINORMAL0;
};

struct OutputBuildVS
{
	float4	posOut		: POSITION0;
    float2  texOut		: TEXCOORD0;
	float3	worldPosOut : TEXCOORD1;
	float3	normOut		: TEXCOORD2;
	float   depth		: TEXCOORD3;
};

struct OutputBuildPS
{
	float4	matOut	 : COLOR0;
	float4	normOut	 : COLOR1;
	float4	posXYOut : COLOR2;
	float4	depthOut : COLOR3;
};

OutputBuildVS vsBuild(InputBuildVS inVS)
{
	OutputBuildVS outVS = (OutputBuildVS)0;

	// Transform vertex to world and screen spaces
	outVS.worldPosOut = mul(float4(inVS.posIn, 1.0f), gWorld);
	float4x4 VP = mul(gView,gProj);
	outVS.posOut = mul(float4(outVS.worldPosOut, 1.0f), VP);
	float4 inView = mul(float4(outVS.worldPosOut, 1.0f),gView);
	outVS.depth = (inView.z / inView.w);
		
	// Provide tex info
	outVS.texOut = inVS.texIn;
	
	// Provide normals
	outVS.normOut = normalize(mul(inVS.normIn, gWorld));
	
	
	return outVS;
};

OutputBuildPS psBuild(OutputBuildVS inVS)
{
	OutputBuildPS outPS = (OutputBuildPS)0;

	// set material info
	float4 diffuseMaterial = tex2D(DiffuseSampler, inVS.texOut);

	outPS.matOut = diffuseMaterial;
	
	// convert normal to texture space [-1;+1] -> [0;1]
	outPS.normOut.xyz = (inVS.normOut * 0.5f) + 0.5f;
	outPS.normOut.w   = 0.0;
	
	// position
	outPS.posXYOut = float4(inVS.worldPosOut.xyz, 0.0f);
	outPS.depthOut = float4(inVS.depth, 0.0f, 0.0f, 0.0f);

	return outPS;
};

struct InputLightPS
{
	float2 texIn	: TEXCOORD0;
};

float4 psLighting(InputLightPS inPS) : COLOR0
{
	float3 ambientMaterial = tex2D(SceneMaterialSampler, inPS.texIn).rgb;
	float3 diffuseMaterial = tex2D(SceneMaterialSampler, inPS.texIn).rgb;
	float3 specularMaterial = tex2D(SceneMaterialSampler, inPS.texIn).a;
	
	// Convert normals back from [0,1] to [-1,+1]
	float3 norm = (tex2D(SceneNormalSampler, inPS.texIn) - 0.5f) * 2.0f;

	float3 pos;
	pos.xy = tex2D(ScenePositionXYSampler, inPS.texIn).xy;
	pos.z = tex2D(ScenePositionXYSampler, inPS.texIn).z;
	float4 endColor = float4(0.0f,0.0f,0.0f,1.0f);
	float3 lightPos = float3(0.0f,8.0f,-2.0f);
	float4 lightDiffuse = float4(0.3f,0.3f,0.3f,1.0f);
	float4 lightSpecular = float4(0.6f,0.6f,0.6f,1.f);
	float4 lightAmbient = float4(0.5f, 0.5f, 0.5f, 1.0f);	
	float3 eyeVec = normalize(gViewInverse[3].xyz - pos);

	float3 lightDir = normalize(lightPos - pos);
	float3 diffuseIntensity = dot(lightDir, norm);
	float3 specularIntensity = pow(max(0, dot(eyeVec, reflect(-lightDir, norm))), 0.8f); // spec power
				
	endColor.rgb += diffuseIntensity * lightDiffuse.xyz * diffuseMaterial +
					specularIntensity * lightSpecular.xyz * specularMaterial +
					lightAmbient * ambientMaterial;

	return endColor;
};


technique BuildPass
{
	pass p0
	{
		VertexShader = compile vs_2_0 vsBuild();
		PixelShader = compile ps_2_0 psBuild();
		
		CullMode = CCW;		
		ZEnable = True;
		ZWriteEnable = True;
		ZFunc = Less;
		StencilEnable = True;
		StencilFunc   = Always;
		StencilPass   = Replace;
		StencilRef    = 1;
		AlphaBlendEnable = False;
		AlphaTestEnable = False;		
		ColorWriteEnable = Red | Green | Blue | Alpha;
	}
}

technique LightPass
{
    pass p0
	{
		VertexShader = NULL;
		PixelShader = compile ps_2_0 psLighting();
		
		CullMode = None;
		FillMode = Solid;
		ZEnable = False;
		StencilEnable = False;
		AlphaBlendEnable = True;
		Srcblend = One;
		Destblend = One;
		AlphaTestEnable = False;		
		ColorWriteEnable = Red | Green | Blue | Alpha;
	}
}
